package jitas.core;

import jitas.exception.*;
import jitas.util.Settings;
import jitas.xml.parser.AllDomainsParser;
import jitas.xml.parser.DomainParser;
import jitas.xml.writer.AllDomainsWriter;
import jess.JessException;
import jess.Rete;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedMap;

/**
 * A domain stores all the material related to a particular tutor subject, that
 * is not specific to a subdomain. Note: Problems are specific to a subdomain.
 * 
 * @author	DeathMarch (c314g2)
 * @version	08 October 2007
 */
public class Domain {
	private String name, longName, description, type, subdomainTerm;
	private boolean userCreationAllowed = true;
	Set<Subdomain> subdomains;
	private Rete rules;
	
	private ProblemSelectionAlgorithm problemSelectionAlgorithm;
	private Feedback customFeedback;
	
	/**
	 * Constructor -This is called from DOMDomainParser to set all of the variables
	 * @param name The name of the domain
	 * @param longName The more detailed name of the domain
	 * @param description The description of the domain
	 * @param userCreation Indicate whether the user is allowed to be created in the domain
	 * @param type The type of domain
	 * @param subdomainTerm The term used to describe the nature of each subdomain
	 * @param subdomains A Set of subdomains contained within this Domain
	 */
	public Domain(String name, String longName, String description, boolean userCreation, String type, String subdomainTerm, Set<Subdomain> subdomains) {
		this.name = name;
		this.longName = longName;
		this.description = description;
		userCreationAllowed = userCreation;
		this.type = type;
		this.subdomainTerm = subdomainTerm;
		this.subdomains = subdomains;
	}
	
	/**
	 * Creates a new Subdomain and adds this to the Set of Subdomains
	 * @param name The name of the new subdomain
	 * @param desc Description for the new Subdomain
	 */
	public void addSubdomain(String name, String desc) {
		Subdomain s = new Subdomain(name, desc, this.name);
		subdomains.add(s);
	}

	/**
	 * Allows read only access to the description of the domain
	 * @return The "private" variable description
	 */
	public String getDescription() {
		return new String(description);
	}
	
	/**
	 * Get the name of the domain
	 * @return A copy of the Name string.
	 */
	public String getName() {
		return new String(name);
	}
	
	/**
	 * Get all the subdomains related to this domain
	 * @return An unmodifiable copy of the set of subdomains
	 */
	public Set<Subdomain> getSubdomains() {
		return Collections.unmodifiableSet(subdomains);
	}
	
	/**
	 * Get the specific subdomain by its name
	 * @param name The name of the subdomain to be searched for
	 * @return s The relevant subdomain
	 * @throws SubdomainNotFound
	 * @throws ProblemsNotFoundException 
	 */
	public Subdomain getSubdomain(String name) throws SubdomainNotFoundException {
		for (Subdomain s: subdomains) {
			if (s.getName().equals(name)) {
				return s;
			}
		}	
		throw new SubdomainNotFoundException("Domain.getSubdomain: Can't find the subdomain named: " + name);
	}
	
	/**
	 * Get a description of each subdomains 
	 * @return A copy of the subdomain term.
	 */
	public String getSubdomainTerm() {
		return new String(subdomainTerm);
	}

	/**
	 * Get the type of this domain
	 * @return A copy of the type of the domain.
	 */
	public String getType() {
		return new String(type);
	}
	
	/**
	 * Get the RuleBase for this domain
	 * @return The RuleBase
	 */
	public Rete getRules() {
		return rules;
	}
	
	/**
	 * Set the RuleBase for the domain
	 * @param rules The RuleBase
	 */
	public void setRules(Rete rules) {
		this.rules = rules;
	}
	
	/**
	 * Loads a domain-specific problem-selection algorithm
	 */
	@SuppressWarnings("unchecked")
	public void loadSpecificAlgorithm(String className){
		String fileName = Settings.getInstance().getAlgorithmFileName(this.name);
		Class<ProblemSelectionAlgorithm> cl = null;
		try {
			URL url = (new File(fileName)).toURL();
			URLClassLoader loader = new URLClassLoader(new URL[] { url });
			cl = (Class<ProblemSelectionAlgorithm>) loader.loadClass(className);
			problemSelectionAlgorithm = (ProblemSelectionAlgorithm) cl.newInstance();
		} catch (Exception e) {
			System.err.println("WARNING: When trying to load custom problem algorithm, got exception: " + e.toString());
			e.printStackTrace();
			problemSelectionAlgorithm = null;
		}
		
	}
	
	/**
	 * Return custom problem selection algorithm. If no custom algorithm exists, return 
	 * default algorithm.
	 */
	public ProblemSelectionAlgorithm getSelectionAlgorithm() {
		if (problemSelectionAlgorithm == null) {
			return new DefaultSelectionAlgorithm();
		} else {
			return problemSelectionAlgorithm;
		}
	}
	
	/**
	 * Loads a domain-specific feedback object
	 */
	@SuppressWarnings("unchecked")
	public void loadSpecificFeedback(String className){
		String fileName = Settings.getInstance().getFeedbackObjectFile(getName());
		Class<Feedback> cl = null;
		try {
			URL url = (new File(fileName)).toURL();
			URLClassLoader loader = new URLClassLoader(new URL[] { url });
			cl =  (Class<Feedback>) loader.loadClass(className);
			customFeedback = (Feedback) cl.newInstance();
		} catch (Exception e) {
			System.err.println("WARNING: When trying to load custom feedback object, got exception: " + e.toString());
			customFeedback = null;
		}
	}
	
	/**
	 * Return the custom feedback object for this domain. If no custom feedback
	 * exists, return default.
	 * @return A Feedback object
	 */
	public Feedback getFeedbackObject() {
		if (customFeedback == null) {
			return new SimpleFeedback();
		} else {
			return customFeedback;
		}
	}
	
	/**
	 * Returns whether or not we are allowed to add users to this domain.
	 * If there is no listed valid users we assume we can add them
	 * @return True if we are allowed to add users to this domain.
	 */
	public boolean userCreationAllowed() {
		return userCreationAllowed;
	}
	
	/**
	 * Read the rules for this domain in from clp files. Note: this does not
	 * read in subdomain rules.
	 * @throws JessException 
	 */
	public void readRules() throws JessException{
		rules = new Rete();
		String path1 = Settings.getInstance().getSemanticConstraintsPath(name);
		jess.Batch.batch(path1, rules);
		String path2 = Settings.getInstance().getSyntaxConstraintsPath(name);
		jess.Batch.batch(path2, rules);
	}
		
	/**
	 * Standard equals override, based on the Domain name
	 * @return true if the objects are equal
	 */
	public boolean equals(Object object) {
		if (object instanceof Domain) {
			Domain other = (Domain) object;
			return this.getName().equals(other.getName());
		} else {
			return false;
		}
	}
	
	/**
	 * Standard compareTo override, based on the Domain name
	 * @param object the Object to be compared
	 * @return a negative integer, zero, or a positive integer as this object
	 * is less than, equal to, or greater than the specified object.
	 * @throws ClassCastException
	 */
	public int compareTo(Object object) throws ClassCastException {
		if (object instanceof Domain) {
			Domain other = (Domain) object;
			return this.getName().compareTo(other.getName());
		} else {
			throw new ClassCastException();
		}
	}
	
	/**
	 * Standard hashCode override, based on the Domain name
	 */
	public int hashCode() {
		return getName().hashCode();
	}

	/**
	 * Get the long name for the domain
	 * @return longName The long name
	 */
	public String getLongName() {
		return longName;
	}

	/**
	 * Set the long name for the domain
	 * @param longName The new long name
	 */
	public void setLongName(String longName) {
		this.longName = longName;
	}

	/**
	 * Gets the set of subdomain names for this domain
	 * @return The set of subdomain names
	 */
	public Set<String> getSubdomainNames() {
		Set<String> names = new HashSet<String>();
		for(Subdomain sd : subdomains) {
			names.add(sd.getName());
		}
		return null;
	}
	
	/**
	 * Load all the subdomains for this domain.
	 * @throws ProblemsNotFoundException If some problems cannot be found for a subdomain
	 */
	public void loadSubdomains() throws ProblemsNotFoundException {
		for(Subdomain sd : subdomains) {
			sd.loadProblems();
		}
	}
	
	/**
	 * Get all the problems for a given subdomain
	 * @param subdomainName The name of the subdomain
	 * @return The problems of the given subdomain
	 * @throws SubdomainNotFoundException
	 */
	public SortedMap<Integer, Problem> getProblems(String subdomainName) throws SubdomainNotFoundException {
		Subdomain subdomain = getSubdomain(subdomainName);
		return subdomain.getProblems();
	}
	
	/**
	 * Get the Solution of a problem, given the subdomain name and problem ID
	 * @param subdomainName The name of the subdomain the problem belongs to
	 * @param problemID The ID of the problem for which the solution is required
	 * @return The Solution to the problem
	 * @throws SubdomainNotFoundException If no subdomain exists with the given name
	 */
	public Solution getSolution(String subdomainName, Integer problemID) throws SubdomainNotFoundException {
		Subdomain subdomain = getSubdomain(subdomainName);
		return subdomain.getSolution(problemID);
	}

	/**
	 * Get the next problem (as selected by the system) from a subdomain
	 * @param student The Student who wants the problem
	 * @param domain The domain
	 * @param problems A sollection of problems to select a problem from
	 * @param subdomainName The name of the subdomain
	 * @return A problem as recommened by the system
	 */
	public Problem getNextProblem(Student student, Domain domain, SortedMap<Integer, Problem> problems, String subdomainName) {
		return getSelectionAlgorithm().getNextProblem(student, domain, problems, subdomainName);
	}

	/**
	 * Add a domain to the global domains list in XML
	 * @param domain
	 */
	public static void addDomain(Domain domain) {
		String path = Settings.getInstance().getAllDomainsFileName();
		AllDomainsWriter w = new AllDomainsWriter(domain, path);
		w.addDomain();
	}

	/**
	 * Get all the domain names from the XML file
	 * @return A set of domain names
	 * @throws IOException If an IO exception occurs while trying to read the XML file
	 */
	public static Set<String> getDomainNames() throws IOException {
		AllDomainsParser p = new AllDomainsParser();
		return p.processTree();
	}

	/**
	 * Load a domain from XML
	 * @param domainName The name of the domain to laod
	 * @return The loaded Domain
	 * @throws DomainNotFoundException If a Domain with the given name cannot be found
	 */
	public static Domain loadDomain(String domainName) throws DomainNotFoundException {
		DomainParser p = new DomainParser(domainName);
		return p.processTree();
	}
}