package jitas.core;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

/**
 * Abstract Feedback class. Should be subclassed specifically for 
 * different domains.
 * 
 * @author	DeathMarch (c314g2)
 * @version	18 August 2007
 */
public interface Feedback {

	/**
	 * Add a message to the Errors, with no additional data.
	 * @param s The message to add
	 */
	public void addMessage(String s);
	
	/**
	 * Add a message to the Errors, with a Map of additional data.
	 * @param s The message to add
	 * @param map The map of additional data
	 */
	public void addMessage(String s, Map<String, String> map);
	
	/**
	 * Adds additional data to a message. If the message already exists, 
	 * the key-value pair is added to the additional data for that message.
	 * If the message doesn't exist, a new error Map entry is made, with
	 * additional data of the key-value pair.
	 * @param s The existing message
	 * @param key The key for the additional data
	 * @param value The value for the additional data
	 */
	public void addAdditionalData(String s, String key, String value);
	
	/**
	 * Get the feedback messages, ignoring any additional data
	 * @return A set of feedback messages
	 */
	public Set<String> getMessages();
	
	/**
	 * Get any additional data associated with a given message.
	 * @param message The message you want the additional data of
	 * @return A map of additional data.
	 */
	public Map<String, String> getAdditionalData(String message);
	
	/**
	 * Get all the errors, including all additional data
	 * @return A Map<String, Map<String, String>> of messages-additional data pairs.
	 */
	public Map<String, Map<String, String>> getErrors();
	
	/**
	 * Add the name of a violated constraint to the collection of violated constraints.
	 * @param name The name of the constraint to add
	 */
	public void addViolatedConstraint(String name);
	
	/**
	 * Get the collection of violated constraint names
	 * @return The violated constraints names
	 */
	public Collection<String> getViolatedConstraints();

	/**
	 * Get the collection of satisfied constraint names
	 * @return The satisfied constraint names
	 */
	public Collection<String> getSatisfiedConstraints();
	
	/**
	 * Add the name of a satisfied constraint to the collection of violated constraints.
	 * @param name  The name of the constraint to add
	 */
	public void addSatisfiedConstraint(String name);
	
	/**
	 * Get the Set of relevant constraints. This is just a union of all 
	 * satisfied and violated constraints, with duplicates removed.
	 * @return The set of relevant constraints
	 */
	public Set<String> getRelevantConstraints();
}
