package jitas.core;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import jitas.exception.UserNotFoundException;
import jitas.util.Settings;
import jitas.xml.writer.StudentModelWriter;

/**
 * The student model is a model of the student knowledge for a particular domain
 * 
 * @author	DeathMarch (c314g2)
 * @version	05 July 2007
 */
public class StudentModel {
	private String userName;
	private int currentProblemID = 0;
	private SortedMap<String, String> problemsStatus;
	private double rating;
	private Collection<String> satisfiedConstraints;
	private Collection<String> violatedConstraints;
	private Set<String> solvedProblems;
	
	/**
	 * Constructor.
	 * @param userName The students userName.
	*/ 
	public StudentModel(String userName){
		this.userName = userName;		
		solvedProblems = new HashSet<String>();
		problemsStatus = new TreeMap<String, String>();
		rating= 0.50;
		satisfiedConstraints = new HashSet<String>();
		violatedConstraints = new HashSet<String>();
	}
	
	/**
	 * Constructor.
	 * @param userName The students userName.
	 * @param solvedProblems A Map of solved problems
	 * @param problemsStatus A Map of problem status'
	*/ 
	public StudentModel(String userName, Set<String> solvedProblems, SortedMap<String, String> problemsStatus){
		this.userName = userName;	
		rating = 0.50;
		this.solvedProblems = solvedProblems;
		this.problemsStatus = problemsStatus;
		satisfiedConstraints = new HashSet<String>();
		violatedConstraints = new HashSet<String>();
	}	
	
	/**
	 * Get the userName of this user.
	 * @return A copy of the username
	 */
	public String getUserName() {
		return new String(userName);
	}
	
	/**
	 * Get the Problem the student is currently working on
	 * @return currentProblem The Problem the student is working on
	 */
	public int getCurrentProblemID() {
		return currentProblemID;
	}
	
	/**
	 * Gets the next problem that hasn't yet been solved by the student
	 * from the set(map) of problems, based on id. This also sets the current problem
	 * @return The next problem to be solved by the user 
	 */
	public Problem getNextProblem(SortedMap<Integer, Problem> problems, String subdomainName) {
		boolean b = false;	
		if (currentProblemID == 0) {
			setCurrentProblem(1);
			return problems.get(1);
		}
		int i = currentProblemID + 1;
		String key = subdomainName + "-" + i;
		/* step through the list until we find one we haven't completed */
		while (solvedProblems.contains(key)) {
			i++;
			key = subdomainName + "-" + i;
			if (i > problems.size() - 1) {
				if (!b) {
					i = 0; // start looking from the start
					b = true;
				} else return null; //all the problems have been done
			}
		}
		Problem p = problems.get(i);
		if (p != null) currentProblemID = p.getId();
		return p;
	}
	
	/**
	 * Gets a problem with a given ID from a collection of problems
	 * @param problems The problems to choose from
	 * @param problemNumber The ID of the wanted problem
	 * @return The Problem
	 */
	public Problem getNextProblem(SortedMap<Integer, Problem> problems, int problemNumber) {
		setCurrentProblem(problemNumber);
		return problems.get(problemNumber);
	}
	
	/**
	 * Add a problem to the collection of solved problems. for a subdomain
	 * @param problemID The ID of the solved problem
	 * @param subdomainName The subdomain name
	 */
	public void addSolvedProblem(int problemID, String subdomainName) {
		solvedProblems.add(subdomainName + "-" + problemID);
	}
	
	/**
	 * Get all solved problems
	 * @return solvedProblems All the problems that the user has completed.
	 */
	public Set<String> getSolvedProblems(){
		return solvedProblems;
	}
	
	/**
	 * Get all solved problems of a particular subdomain
	 * @param subdomainName The name of the subdomain
	 * @return solvedProblems All the problems that the user has completed of a subdomain.
	 */
	public Set<Integer> getSolvedProblems(String subdomainName){
		Set<Integer> set = new HashSet<Integer>();		
		for(String s : solvedProblems) {
			if (s.length() > 0 && s.substring(0, s.indexOf('-')).equals(subdomainName)) {
				set.add(Integer.parseInt(s.substring(s.indexOf('-')+1)));
			}
		}
		return set;
	}
	
	/**
	 * Returns a new empty model with just username set
	 * @param username The username of the Student
	 * @return A new StudentModelObject
	 */
	public static StudentModel emptyModelFactory(String username) {
		return new StudentModel(username);
	}
	
	/**
	 * Set the status of a problem
	 * @param problemID The id of the problem
	 * @param status The status must be one of "solved", "attempted" or "not attempted"
	 */
	public void setProblemStatus(String subdomain, int problemID, String status) {
		String key = subdomain + "-" + problemID;
		if (status.equals("solved") || status.equals("attempted") || status.equals("not attempted")) {
			problemsStatus.put(key, status);
		} else
			throw new IllegalArgumentException("Status must be \"solved\", \"attempted\" or \"not attempted\"");
	}
	
	/**
	 * Get the status for a problem.
	 * @param problemID The id of the problem
	 * @return r The status has to be one of "solved", "attempted" or "not attempted"
	 */
	public String getProblemStatus(String subdomain, int problemID) {
		String r = problemsStatus.get((subdomain + "-" + problemID));
		if (r == null) {
			r = "not attempted";
		}
		return r;
	}
	
	/**
	 * Get the map of problem status'
	 * @return The map
	 */
	public SortedMap<String, String> getAllProblemsStatus() {
		return problemsStatus;
	}

	/**
	 * Set current problem to problem one
	 * @param currentSubdomain
	 */
	public void changedSubdomain(String currentSubdomain) {
		setCurrentProblem(1);
	}
	
	/**
	 * Get The ability rating of the student
	 * @return rating The Ability rating of the student
	 */
	public double getRating() {
		return rating;
	}
	
	/**
	 * Set The ability rating of the student
	 */
	public void setRating(double rating) {
		this.rating = rating;
	}
	
	/**
	 * Set the Problem the student is currently working on
	 * @param currentProblemID The ID of the Problem the student wants to work on
	 */
	public void setCurrentProblem(int currentProblemID) {
		this.currentProblemID = currentProblemID;
	}
	
	/**
	 * Get the collection of violated constraint names
	 * @return violatedConstraints The set of violated constraints names
	 */
	public Collection<String> getViolatedConstraints() {
		return violatedConstraints;
	}

	/**
	 * Get the collection of satisfied constraint names
	 * @return satisfiedConstraints The set of satisfied constraint names
	 */
	public Collection<String> getSatisfiedConstraints() {
		return satisfiedConstraints;
	}
	
	/**
	 * Set the collection of violated constraint names
	 * @param violatedConstraints The violated constraints names
	 */
	public void setViolatedConstraints(Collection<String> violatedConstraints) {
		this.violatedConstraints = violatedConstraints;
	}
	
	/**
	 * Adds the violated constraint to the collection of violated constraint names
	 * @param violatedConstraints The violated constraints names
	 */
	public void addViolatedConstraints(Collection<String> violatedConstraints) {
		this.violatedConstraints.addAll(violatedConstraints);
	}
	
	/**
	 * Adds the satisfied constraint to the collection of satisfied constraint names
	 * @param satisfiedConstraints The satisfied constraints names
	 */
	public void addSatisfiedConstraints(Collection<String> satisfiedConstraints) {
		this.satisfiedConstraints.addAll(satisfiedConstraints);
	}
	
	/**
	 * Set the collection of satisfied constraint names
	 * @param satisfiedConstraints The satisfied constraint names
	 */
	public void setSatisfiedConstraints(Collection<String> satisfiedConstraints) {
		this.satisfiedConstraints = satisfiedConstraints;
	}
	
	/**
	 * Standard equals override, based on the userName
	 * @return true if the objects are equal
	 */
	public boolean equals(Object object) {
		if (object instanceof StudentModel) {
			StudentModel other = (StudentModel) object;
			return this.getUserName().equals(other.getUserName());
		} else {
			return false;
		}
	}
	
	/**
	 * Standard compareTo override, based on the userName
	 * @param object the Object to be compared
	 * @return a negative integer, zero, or a positive integer as this object
	 * is less than, equal to, or greater than the specified object.
	 * @throws ClassCastException
	 */
	public int compareTo(Object object) throws ClassCastException {
		if (object instanceof StudentModel) {
			StudentModel other = (StudentModel) object;
			return this.getUserName().compareTo(other.getUserName());
		} else {
			throw new ClassCastException();
		}
	}
	
	/**
	 * Standard hashCode override, based on the userName
	 */
	public int hashCode() {
		return getUserName().hashCode();
	}

	/**
	 * Increment the rating by some value
	 * @param increment The value to increase the rating by
	 */
	public void incrementRating(double increment) {
		setRating(getRating()+increment);
	}

	/**
	 * Set the status of the current problem for a subdomain
	 * @param subdomainName The subdomain name
	 * @param status The status
	 */
	public void setCurrentProblemStatus(String subdomainName, String status) {
		String key = subdomainName + "-" + currentProblemID;
		problemsStatus.put(key, status);
		
	}

	/**
	 * Save this model to XML
	 * @param domainName The name of the domain this model is for
	 */
	public void save(String domainName) {
		String path = Settings.getInstance().getStudentModelPath(domainName);
		StudentModelWriter smw = new StudentModelWriter(this, path);
		try {
			smw.persistStudentModel();
		} catch (UserNotFoundException e) {
			System.err.println("WARNING when trying to save model: "+e.getMessage()+" adding new model");
			smw.addStudentModel();
		}
	}
}
