package jitas.core;

import jitas.exception.*;
import jitas.util.Logger;
import jitas.util.Settings;
import jitas.xml.parser.*;
import java.util.*;

/**
 * A subdomain object. Subdomains contain problems and solutions, and also
 * can potentially have specific constraints relevant to the subdomain.
 * A domain can have many subdomains, but must have at least one.
 * 
 * @author	DeathMarch (c314g2)
 * @version	13 September 2007
 */
public class Subdomain implements Comparable<Subdomain>{
	
	private String name, description,domainName;
	private SortedMap<Integer, Problem> problems;
	
	/**
	 * Constructor -- This is called when the DOM Domain Parser parses the domain
	 * @param name The subdomain name
	 * @param description The description of the subdomain
	 * @param domainName The owning domain name
	 */
	public Subdomain(String name, String description, String domainName) {
		this.name= name;
		this.description = description;
		this.domainName = domainName;
		problems = new TreeMap<Integer, Problem>();
	}
	
	/**
	 * Returns the name of the subdomain. Names must be unique within each domain.
	 * @return The name of the subdomain
	 */
	public String getName() {
		return new String(name);
	}

	/**
	 * Returns the description of the subdomain
	 * @return The description of the subdomain
	 */
	public String getDescription() {
		return new String(description);
	}
	
	/**
	 * Returns the map of problems associated with this subdomain
	 * @return problems The map of problems
	 */
	public SortedMap<Integer, Problem> getProblems() {
		return problems;
	}
	
	/**
	 * Gets a problem specified by the problem number
	 * @param problemNumber The number of the problem you want
	 * @return The problem of the specified problem number
	 */
	public Problem getProblem(int problemNumber) {
		return problems.get((Integer)problemNumber);
	}
	
	public Solution getSolution(int problemNumber) {
		return getProblem(problemNumber).getSolution();
	}
	
	/**
	 * Adds a problem to the List
	 * @param p The problem which is to be stored.
	 * @return True if the problem was added
	 */
	public boolean addProblem(Problem p)throws ProblemAlreadyExistsException {	
		if(p == null)
			return false;		
		try {			
			if(problems.containsKey((Integer)p.getId())) {	
				String error = "Subdomain.addProblem:attempted to add a duplicate Problem to the List.";
				throw new ProblemAlreadyExistsException(error);
			}	
			problems.put((Integer)p.getId(), p) ;
			return true;
		}catch (ProblemAlreadyExistsException e) {
			throw e;
		}catch (Exception ex) {
			System.out.println(ex.toString());
			System.err.print("Subdomain.addProblem: Could not add problem to SortedMap in Subdomain.");
		}
		return false;
	}
	
	/**
	 * Load problems for the subdomain from the correct xml file
	 * eg domains/<domain-name>/<subdomain-name>-problems.xml
	 * Note: General, (ie non subdomain-specific) problems are no longer supported;
	 * every problem must be in a subdomain.
	 */
	public void loadProblems() throws ProblemsNotFoundException {
		try {
			String path = Settings.getInstance().getRootPath() + domainName + "/" + name + "-" + "problems.xml";
			ProblemParser parser = new ProblemParser(path);
			problems = parser.processTree();
		} catch (Exception e) {
			Logger.addEntry("Could not find problems for subdomain: " + name);
		}
	}

	/**
	 * Standard equals override, based on the Subdomain name
	 * @return True if the objects are equal
	 */
	public boolean equals(Object object) {
		if (object instanceof Subdomain) {
			Subdomain other = (Subdomain) object;
			return this.getName().equals(other.getName());
		} else {
			return false;
		}
	}
	
	/**
	 * Standard compareTo override, based on the Subdomain name
	 * @param object The Object to be compared
	 * @return A negative integer, zero, or a positive integer as this object
	 * is less than, equal to, or greater than the specified object.
	 * @throws ClassCastException
	 */
	public int compareTo(Subdomain object) throws ClassCastException {
		if (object instanceof Subdomain) {
			Subdomain other = (Subdomain) object;
			return this.getName().compareTo(other.getName());
		} else {
			throw new ClassCastException();
		}
	}
	
	/**
	 * Standard hashCode override, based on the Subdomain name
	 */
	public int hashCode() {
		return getName().hashCode();
	}
	
	/**
	 * Standard toString override
	 */
	public String toString() {
		return "Subdomain: " + name + ", " + description; 
	}
}
