package jitas.util;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;

import jess.JessException;
import jess.RU;
import jess.Rete;
import jess.Value;
import jess.ValueVector;
import jitas.core.*;
import jitas.exception.*;
/**
 * Class for evaluating students solutions
 * @author DeathMarch(c314g2)
 * @version 08 October 2007
 */
public class SolutionEvaluator {
	
	/**
	 * Evaluate a students solution. For more details see the comment on 
	 * TutorInterface.submitSolution
	 * @param studentSolution The solution object
	 * @param student The Student
	 * @param domain The Domain
	 * @param subdomainName The name of the subdomain
	 * @param external Whether it's a student submitting or problem selection using the evaluator
	 * @return The Feedback object
	 * @throws StudentModelNotFoundException If theres no student model for the Student in the domain
	 * @throws JessException If theres a problem with JESS
	 * @throws InvalidSolutionException If the solution is invalid
	 * @throws UserNotFoundException If the student is not found
	 */
	public static Feedback evaluateSolution(Solution studentSolution, Student student, Domain domain, String subdomainName, boolean external) 
	throws StudentModelNotFoundException, JessException, InvalidSolutionException, UserNotFoundException{
		
		Feedback feedback = domain.getFeedbackObject();
		String domainName = domain.getName();
		Integer currentProblemID = student.getCurrentProblemID(domainName);
		String status = student.getProblemStatus(domainName, currentProblemID);
		
		if(external){
			student.setCurrentProblemStatus(domainName, "attempted");
		}
		/* it would be good if we could remove this line, as reading the rules in everytime can't be fast...*/
		//domain.readRules();
		Rete rules = domain.getRules();
		rules.reset();
		
		/* add the subdomain specific rules */
		String path1 = Settings.getInstance().getSemanticConstraintsPath(domainName, subdomainName);
		try {
			jess.Batch.batch(path1, rules);
		} catch (JessException e) {}
		String path2 = Settings.getInstance().getSyntaxConstraintsPath(domainName, subdomainName);
		try {
			jess.Batch.batch(path2, rules);
		} catch (JessException e) {}
						
		Solution idealSolution;
		try {
			idealSolution = domain.getSolution(subdomainName, currentProblemID);
		} catch (SubdomainNotFoundException e) {
			throw new InvalidSolutionException("ERROR getting solution: subdomain "+subdomainName+" not found");
		}
			
		/* try to assert student solution */
		assertSolution(rules, "SS", studentSolution, domainName);
		
		/* try to assert ideal solution (this shouldn't fail) */
		assertSolution(rules, "IS", idealSolution, domainName);
		
		rules.store("FEEDBACK", feedback);
		rules.run();
		if (feedback.getViolatedConstraints().size() == 0 && external) {
			if(status.equals("not attempted")){
				student.incrementRating(domainName, 0.05);
			}
			
			student.setCurrentProblemStatus(domainName, "solved");
			student.addSolvedProblem(domainName, subdomainName, currentProblemID);
		} else {
			if(status.equals("not attempted") && external){
				student.incrementRating(domainName, -0.02);
			}
			student.addViolatedConstraints(domainName, feedback.getViolatedConstraints());
		}
		student.addSatisfiedConstraints(domainName, feedback.getSatisfiedConstraints());
        
        student.saveStudentModel(domainName);
		
		rules.clearStorage();
		
		return feedback;
	}
	
	/*Assert solution against rules*/
	private static void assertSolution(Rete rules, String type, Solution s, String domainName) 
	throws InvalidSolutionException, JessException {
		try {
		//assert empty solution
			rules.eval("(assert (" + type + "))");
			int p = 0; //pointer in shortString
			int c = 0; //pointer in solution content
			String sc = s.getContent();
			while(c < sc.length()-1) {
				p = 0;
				String currentSlot = sc.substring(c);
				currentSlot = currentSlot.substring(0, currentSlot.indexOf(")")+1);
				c+=currentSlot.indexOf(")")+1;
				
				if (currentSlot.charAt(p) != '(')
					throw new InvalidSolutionException("Error asserting " + type + ": " + s.getContent());
				
				String slotName = currentSlot.substring(1, currentSlot.indexOf(" "));
				
				int start = currentSlot.indexOf(" ");
				int end = currentSlot.indexOf(")");
				
				List<String> strings = new ArrayList<String>();
				Class<JESSParser> cl = loadSpecificJESSParser(domainName);
				try {
					Object o = cl.newInstance(); 
					strings = ((JESSParser)o).tokenise(currentSlot.substring(start, end));
				} catch (Exception e) {
					Logger.addEntry("Using default JITAS JESSParser.");
					DefaultJESSParser djp = new DefaultJESSParser();
					strings = djp.tokenise(currentSlot.substring(start, end));
				}
				
				ValueVector vv = new ValueVector();
				for(String str : strings) {
					Value v = new Value(str, RU.MULTISLOT);
					vv.add(v);
				}
				
				String evalString = "(assert (" + type + " (" + slotName + " " + vv + ")))";
				rules.eval(evalString);
			}
		} catch (Exception e) {
			throw new InvalidSolutionException("Error asserting " + type + ": " + s.getContent());
		}
	}
	
	/**
	 * Loads a domain-specific system-chosen algorithm
	 * @return cl Class loaded from the specified directory 
	 */
	@SuppressWarnings("unchecked")
	public static Class<JESSParser> loadSpecificJESSParser(String domainName){
		String fileName = Settings.getInstance().getJESSParserFile(domainName);
		Class<JESSParser> cl = null;
		try {
			URL url = (new File(fileName)).toURL();
			URLClassLoader loader = new URLClassLoader(new URL[] { url });
			cl =  (Class<JESSParser>) loader.loadClass("SpecificJESSParser");
		} catch (MalformedURLException e) {
			return null;
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (Exception e){
			e.printStackTrace();
		}
		return cl;
	}
}
