package jitas.xml.parser;

import jitas.core.StudentModel;
import jitas.exception.*;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import java.io.*;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * An XML parser to parse a set of Student Models from a given file.
 * There will be one such file for each domain.
 * 
 * @author	DeathMarch (c314g2)
 * @version	04 July 2007
 */
public class StudentModelParser extends XMLParser{
	
	private static final String STUDENT_MODELS_TAG = "student-models";
	private static final String STUDENT_MODEL_TAG = "student-model";
	private static final String USERNAME_TAG = "username";
	private static final String ABILITY_RATING_TAG = "rating";
	private static final String PROBLEMS_STATUS_TAG = "problems-status";
	private static final String SOLVED_PROBLEMS = "solved-problems";
	private static final String CONSTRAINTS_SATISFIED = "constraints-satisfied";
	private static final String CONSTRAINTS_VIOLATED = "constraints-violated";
	//private Set<StudentModel> studentModels = null;
	private Document parsedDoc = null;
	//private Domain domain;//the domain we are loading student-models for
	private String pathName = "";
	private String domainName;
	private String studentName;
	
	/**
	 * The constructor sorts out the arguments, then hands
	 * over the DOM work to the DOM parser; which is used to build 
	 * the DOM tree and then parse the student model docs (.xml). 
	 * @param pathName Pathname to the file
	 * @param domainName The domain name
	 * @param studentName The Students names
	 * @throws StudentModelsNotFoundException If the models for the student weren't found
	 */
	public StudentModelParser(String pathName, String domainName, String studentName) throws StudentModelsNotFoundException {
	
		/* If a domain has just been added to the
		 * folder, then there will be no path to a 
		 * student model, as that file may not have been created yet.
		 * Currently, there will be an exception!
		 */
		this.pathName = pathName;
		this.domainName = domainName;
		this.studentName = studentName;
		try {
        	parsedDoc = super.loadXML(pathName);
        } catch (SAXException se) {
        	System.err.println(se.getMessage());        	
        } catch (FileNotFoundException fnfe) {	
        	fnfe.printStackTrace();
        	throw new StudentModelsNotFoundException("StudentModelParser.Constructor:student models file not found at: " + pathName);
        } catch (IOException ioe) {
        	System.err.println(ioe);
        }
	}
	
	public StudentModel processTree(){
		try {
			return loadStudentModel(domainName, studentName);
		} catch (StudentModelNotFoundException e) {
			// Didn't find a student model for the student, make a new one
			return new StudentModel(studentName, new HashSet<String>(), new TreeMap<String, String>());
		}
	}
	
	/**
	 * The root of the tree is easy to find since the document
	 * itself implements the Node interface...
	 * @return studentModels The set of StudentModels
	 * @throws StudentModelNotFoundException If it cannot find the model for the student
	 */
	public StudentModel loadStudentModel(String domainName, String studentName) throws StudentModelNotFoundException {
		Node root = null;
		//studentModels = new HashSet<StudentModel>();
		SortedMap<String, String> problemsStatus = new TreeMap<String, String>();
		Set<String> solvedProblems = new HashSet<String>();
		Collection<String> constraintSatisfied = new HashSet<String>();
		Collection<String> constraintViolated = new HashSet<String>();
		String userName = "";
		double rating = 0.5;
		boolean correctStudent = false;
		
		if(super.isFileEmpty(pathName)){
			/* Create a student model*/
	        Element studentModel = parsedDoc.createElement("student-models");
	        /*Append the student model element to the root element*/
			parsedDoc.appendChild(studentModel);
			super.writeToXML(parsedDoc, pathName);
			return null;
		}
		/* the first node is a document node */
		root = parsedDoc.getFirstChild();
		if (root.getNodeName().equals(STUDENT_MODELS_TAG)) {
			
			/* for each child */
			for (Node n = root.getFirstChild(); n != null; n = n.getNextSibling()) {		
				if (n.getNodeName().equals(STUDENT_MODEL_TAG)) {
					
					for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {				
						if(child.getNodeName().equals(USERNAME_TAG)) {
							userName = child.getFirstChild().getNodeValue();//hope this works :)
							if(userName.equals(studentName)) correctStudent = true;	
						}
						if(child.getNodeName().equals(PROBLEMS_STATUS_TAG)) {
							try {
								String str = child.getFirstChild().getNodeValue();
								str = str.substring(1, str.length()-1);
								String[] parts = str.split(", ");
								for (int i = 0; i < parts.length; ++i) {
									String[] entry = parts[i].split("=");
									problemsStatus.put(entry[0], entry[1]);
								}
							} catch (Exception e) {
								problemsStatus = new TreeMap<String, String>();
							}
						}
						if(child.getNodeName().equals(SOLVED_PROBLEMS)) {
							try {
								String str = child.getFirstChild().getNodeValue();
								str = str.substring(1, str.length()-1);
								String[] parts = str.split(", ");
								for (int i = 0; i < parts.length; ++i) {
									//System.out.println(parts[i]);
									solvedProblems.add(parts[i]);
								}
							} catch (Exception e) {
								//System.err.println(e);
								//solvedProblems = new HashSet<Integer>();
							}
						}
						if(child.getNodeName().equals(CONSTRAINTS_SATISFIED)) {
							if(child.getFirstChild() != null){
								child = child.getFirstChild().getNextSibling();
								for(Node grandChild = child.getFirstChild(); grandChild != null; grandChild = grandChild.getNextSibling()){ 
									constraintSatisfied.add(grandChild.getNodeValue());
								}
							}
						}
						if(child.getNodeName().equals(CONSTRAINTS_VIOLATED)) {
							if(child.getFirstChild() != null){
								child = child.getFirstChild().getNextSibling();
								for(Node grandChild = child.getFirstChild(); grandChild != null; grandChild = grandChild.getNextSibling()){ 
									constraintViolated.add(grandChild.getNodeValue());	
								}
							}
						}
						if(child.getNodeName().equals(ABILITY_RATING_TAG)) {
							String s = child.getFirstChild().getNodeValue();
							rating = Double.parseDouble(s);
						}
					}
					if(correctStudent) {
						StudentModel sm = new StudentModel(userName, solvedProblems, problemsStatus);
						sm.setSatisfiedConstraints(constraintSatisfied);
						sm.setViolatedConstraints(constraintViolated);
						sm.setRating(rating);
						return sm;
					}
				}  	
			}
		}
		throw new StudentModelNotFoundException("Could not find student model for: "+studentName+" in "+domainName);
	}	
}
