package jitas.xml.parser;

import jitas.core.Student;
import jitas.exception.*;
import jitas.util.Logger;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import java.io.*;
import java.util.HashSet;
import java.util.Set;

/**
 * An XML parser to parse a set of Student Models from a given file.
 * There will be one such file for each domain.
 * 
 * @author	DeathMarch (c314g2)
 * @version	04 July 2007
 */
public class StudentParser extends XMLParser{
	
	private static final String STUDENTS_TAG = "students";
	private static final String STUDENT_TAG = "student";
	private static final String USERNAME_TAG = "username";
	private static final String PASSWORD_TAG = "password";
	private static final String ALLOWED_DOMAINS_TAG = "allowed-domains";
	private static final String DOMAIN_TAG = "domain";
	private Set<Student> students = null;
	private Document parsedDoc = null;
	private String pathName = "";
	private String rootPath = "";
	private String modelsFileName = "";
	
	/**
	 * The constructor sorts out the arguments, then hands
	 * over the DOM work to the DOM parser; which is used to build 
	 * the DOM tree and then parse the student model docs (.xml).
	 * @param studentPath Path to the students XML file
	 * @param rootPath The rootPath
	 * @param modelsFileName The name of the student models file
	 * @throws FileNotFoundException If the file cannot be found
	 */
	public StudentParser(String studentPath, String rootPath, String modelsFileName) throws FileNotFoundException {
		this.pathName = studentPath;
		this.rootPath = rootPath;
		this.modelsFileName = modelsFileName;
		/* If a domain has just been added to the
		 * folder, then there will be no path to a 
		 * student model, as that file may not have been created yet.
		 * Currently, there will be an exception!
		 */
		try {
        	parsedDoc = super.loadXML(pathName);
        } catch (SAXException se) {
        	System.err.println(se.getMessage());
        } catch (FileNotFoundException fnfe) {	
        	throw new FileNotFoundException("StudentModelParser.Constructor:student models file not found at: " + pathName);
        } catch (IOException ioe) {
        	System.err.println(ioe);
        }
	}
	
	/**
	 * The root of the tree is easy to find since the document
	 * itself implements the Node interface...
	 * @return students The set of students
	 */
	public Set<Student> processTree() {
		Node root = null;
		students = new HashSet<Student>();
		String userName, password;
		userName = password = "";
		Set<String> allowedDomains = new HashSet<String>();
		

		
		if(super.isFileEmpty(pathName)){
			/* Create a student model*/
	        Element studentModel = parsedDoc.createElement("students");
	        /*Append the student model element to the root element*/
			parsedDoc.appendChild(studentModel);
			super.writeToXML(parsedDoc, pathName);
			return null;
		}
		/* the first node is a document node */
		root = parsedDoc.getFirstChild();
		if (root.getNodeName().equals(STUDENTS_TAG)) {
			
			/* for each child */
			for (Node n = root.getFirstChild(); n != null; n = n.getNextSibling()) {		
				if (n.getNodeName().equals(STUDENT_TAG)) {				
					/* for each child (attributes eg username) */
					for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {				
						if (child.getNodeName().equals(USERNAME_TAG)) {
							userName = child.getFirstChild().getNodeValue();
						}else if(child.getNodeName().equals(PASSWORD_TAG)) {
							password = child.getFirstChild().getNodeValue();
						} else if(child.getNodeName().equals(ALLOWED_DOMAINS_TAG)) {						
							for (Node allowedDomain = child.getFirstChild(); allowedDomain != null; allowedDomain = allowedDomain.getNextSibling()) {
								if(allowedDomain.getNodeName().equals(DOMAIN_TAG)) {
									 allowedDomains.add(allowedDomain.getTextContent());									 
								 }
							}
						}					   		
					}
					
					try {
						Student student = new Student(userName, password, allowedDomains);
						
						//load all the student models for that student
						for(String domainName : allowedDomains) {
							String pathName = rootPath + domainName + modelsFileName;
							try {
								StudentModelParser smp = new StudentModelParser(pathName, domainName, userName);
								student.addStudentModel(domainName, smp.processTree());
							} catch (StudentModelsNotFoundException e) {
								System.err.println(e.getMessage());
							}
							
						}
						/* add the model to the list */
						students.add(student);
						/* Now reset allowedDomains to a new HashSet<String> */
						allowedDomains = new HashSet<String>();
					} catch (IllegalUserNameException ex) {
						Logger.addEntry("Got an illegal user name from file:" + pathName);
						Logger.addEntry(ex.toString());
					}
				}
			}  	
		}
		return students;
	}
}
