package jitas.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * A simple implementation of the Feedback class. Messages are stored in a Map, of
 * String-Map pairs. Messages are keys to additional data maps, which contain key-value pairs.
 * This gives a format of:
 * <(message, <(key, value), (key2, value2)>), (message2, <(key, value), (key2, value2)>)>
 *  
 * @author	DeathMarch (c314g2)
 * @version	18 August 2007
 */
public class SimpleFeedback implements Feedback, Serializable {

	private static final long serialVersionUID = -6865414267942946628L;
	private Map<String, Map<String, String>> errors;
	private Collection<String> violatedConstraints;
	private Collection<String> satisfiedConstraints;
	
	/**
	 * Constructor - initialises globals.
	 */
	public SimpleFeedback() {
		violatedConstraints = new ArrayList<String>();
		satisfiedConstraints = new ArrayList<String>();
		errors = new HashMap<String, Map<String, String>>();
	}
	
	/**
	 * Add a message to the Errors, with no additional data.
	 * @param s The message to add
	 */
	public void addMessage(String s) {
		errors.put(s, new HashMap<String, String>());
	}
	
	/**
	 * Add a message to the Errors, with a Map of additional data.
	 * @param s The message to add
	 * @param map The map of additional data
	 */
	public void addMessage(String s, Map<String, String> map) {
		errors.put(s, map);
	}
	
	/**
	 * Adds additional data to a message. If the message already exists, 
	 * the key-value pair is added to the additional data for that message.
	 * If the message doesn't exist, a new error Map entry is made, with
	 * additional data of the key-value pair.
	 * @param s The existing message
	 * @param key The key for the additional data
	 * @param value The value for the additional data
	 */
	public void addAdditionalData(String s, String key, String value) {
		if (errors.containsKey(s)) {
			Map<String, String> map = errors.get(s);
			map.put(key, value);
		} else {
			Map<String, String> newMap = new HashMap<String, String>();
			newMap.put(key, value);
			addMessage(s, newMap);
		}
	}
	
	/**
	 * Get the feedback messages, ignoring any additional data
	 * @return A set of feedback messages
	 */
	public Set<String> getMessages() {
		return errors.keySet();		
	}
	
	/**
	 * Get any additional data associated with a given message.
	 * @param message The message you want the additional data of
	 * @return A map of additional data.
	 */
	public Map<String, String> getAdditionalData(String message) {
		return errors.get(message);
	}
	
	/**
	 * Get all the errors, including all additional data
	 * @return A Map<String, Map<String, String>> of messages-additional data pairs.
	 */
	public Map<String, Map<String, String>> getErrors() {
		return errors;
	}
	
	/**
	 * Add the name of a violated constraint to the collection of violated constraints.
	 * @param name The name of the constraint to add
	 */
	public void addViolatedConstraint(String name) {
		violatedConstraints.add(name);
	}
	
	/**
	 * Get the collection of violated constraint names
	 * @return violatedConstraints The violated constraints names
	 */
	public Collection<String> getViolatedConstraints() {
		return violatedConstraints;
	}

	/**
	 * Get the collection of satisfied constraint names
	 * @return satisfiedConstraints The satisfied constraint names
	 */
	public Collection<String> getSatisfiedConstraints() {
		return satisfiedConstraints;
	}
	
	/**
	 * Add the name of a satisfied constraint to the collection of violated constraints.
	 * @param name The name of the constraint to add
	 */
	public void addSatisfiedConstraint(String name) {
		satisfiedConstraints.add(name);
	}
	
	/**
	 * Get all the relevant constraints of a problem
	 * @return s A set of relevant constraints
	 */
	public Set<String> getRelevantConstraints() {
		Set<String> s = new HashSet<String>();
		s.addAll(violatedConstraints);
		s.addAll(satisfiedConstraints);
		return s;
	}
}
